package GCOptions;

###################################################
#
#  Copyright 2005-2006 Tian
#
#  This file is part of GCstar.
#
#  GCstar is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  GCstar is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with GCstar; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###################################################

use strict;
use Gtk2;

{
    package GCOptionLoader;

    my $DEFAULT_LANG='EN';

    use XML::Simple;
    use GCLang;

    sub new
    {
        my ($proto, $file, $main) = @_;
        my $class = ref($proto) || $proto;
        my $self  = {created => 0};
        
        GCLang::loadLangs;
        
        bless ($self, $class);
        $self->load($file, $main) if $file;
        return $self;
    }

    sub newFromXmlString
    {
        my ($proto, $string) = @_;
        my $class = ref($proto) || $proto;
        my $self  = {created => 0};
        bless ($self, $class);
        if ($string)
        {
            my $xs = XML::Simple->new;
            $self->{options} = $xs->XMLin($string,
                                          SuppressEmpty => 1);
        }
        return $self;
    }

    sub toXmlString
    {
        my $self = shift;

        my $result = "<".$GCData::inlinePreferencesTag.">\n";
        while (my ($key,$value) = each(%{$self->{options}}))
        {
            $result .= "  <$key>$value</$key>\n";
        }
        $result .= "</$GCData::inlinePreferencesTag>";
        return $result;
    }

    sub error
    {
        my ($self, $type) = @_;
        if ($self->{parent})
        {
            $self->{parent}->optionsError($type, $self->{file});
        }
        else
        {
            print "Error performing $type for ".$self->{file}."\n";
        }
    }

    sub setParent
    {
        my ($self, $parent) = @_;
        $self->{parent} = $parent;
    }

    sub load
    {
        my ($self, $file, $main) = @_;

        $self->{file} = $file;

        $self->{options} = undef;

        if (! -e $file)
        {
            open OPTIONS, ">$file" or $self->error('create');
            if ($main)
            {
                $self->{created} = 1;
            
                my $tmpLang = $ENV{LANG};
                $tmpLang =~ s/(..)_.*/\U$1\E/ if $tmpLang;
                $tmpLang = $DEFAULT_LANG if ! $tmpLang;
            
                print OPTIONS 'images='.$ENV{GCS_DATA_HOME}.'/images/
autosave=1
noautoload=0
browser=firefox
player=mplayer
audio=xmms
file=
split=300
width=920
height=640
confirm=1
lang='.$tmpLang.'
status=1
splash=1
toolbar=2
toolbarPosition=0
status=1
transform=1
articles=le,la,les,l,un,une,des,a,the,der,die,das,ein,eine,el,los,una
askImport=0
searchStop=1
alwaysOriginal=0
proxy=
borrowers=
emails=
view=0
columns=3
listBgPicture=1
mailer=Sendmail
from=
subject=GCstar email
smtp=
template=Hello %1,<br/><br/>You borrowed me %2 since %3. I\'d like to get it back shortly.
history=none|none|none|none|none
useRelativePaths=0
useTitleForPics=0
';
            }
            else
            {
                print OPTIONS "\n";
            }
            close OPTIONS;
        }

        open OPTIONS, $file or $self->error('open');
        
        while (<OPTIONS>)
        {
            chomp;
            my @assign =  split m/=/;
            $self->{options}->{$assign[0]} = $assign[1];
        }
                
        close OPTIONS;
        $self->lang($DEFAULT_LANG) if $main && (! exists $GCLang::langs{$self->lang});
    }

    sub getFullLang
    {
        my $self = shift;
        
        (my $lang = $self->lang) =~ s/(.*)/\L$1\E_$1/;
        $lang =~ s/_EN/_US/; # Fix for english
        $lang =~ s/_CS/_CZ/; # Fix for Czech
        $lang .= '.UTF-8';
        $lang = 'sr@Latn' if $lang =~ /^sr/;  # Fix for serbian        
        return $lang;
    }

    sub save
    {
        my $self = shift;

        open OPTIONS, '>'.$self->{file} or $self->error('save');
   
        my %options = %{$self->{options}};
     
        my $line = '';
		my $key;
		my $value;
        while (($key,$value) = each(%options))
        {
            $line = $key.'='.$value;
            print OPTIONS "$line\n";            
        }
        
        close OPTIONS;
    }

    sub exists
    {
        my ($self, $name) = @_;
        
        return exists $self->{options}->{$name};
    }

    sub AUTOLOAD
    {
        my $self = shift;
        my $name = our $AUTOLOAD;
        return if $name =~ /::DESTROY$/;
        $name =~ s/.*?::(.*)/$1/;
        if (@_)
        {
            $self->{options}->{$name} = shift;
        }
        else
        {
            return $self->{options}->{$name};
        }
    }
    
    sub checkPreviousGCfilms
    {
        my ($self, $parent) = @_;
        
        $self->{modelsFactory} = $parent->{modelsFactory};
        my $gcfilmsconf = $ENV{XDG_CONFIG_HOME}.'/gcfilms/gcfilms.conf';

        if ( -e $gcfilmsconf)
        {
            my  $dialog = Gtk2::MessageDialog->new($parent,
                        [qw/modal destroy-with-parent/],
                        'question',
                        'ok-cancel',
                        $parent->{lang}->{GCfilmsImportQuestion});
            
            my $withOptionsCb = new Gtk2::CheckButton($parent->{lang}->{GCfilmsImportOptions});
            my $withDataCb = new Gtk2::CheckButton($parent->{lang}->{GCfilmsImportData});
            $dialog->vbox->pack_start($withOptionsCb, 0, 0, 0);
            $dialog->vbox->pack_start($withDataCb, 0, 0, 0);
            $dialog->vbox->show_all;
            #$parent->{splash}->hide if $parent->{splash};
            my $response = $dialog->run;
            my $withOptions = $withOptionsCb->get_active;
            my $withData = $withDataCb->get_active;
            $dialog->destroy;
            return if $response ne 'ok';
            my $gcfilmsOptions = GCOptionLoader->new($gcfilmsconf, 0);
            $self->importGCfilmsOptions($gcfilmsOptions) if $withOptions;
            $self->importGCfilmsData($gcfilmsOptions) if $withData;            
        }
    }
    
    sub importGCfilmsOptions
    {
        my ($self, $options) = @_;
        
        my @commonOptions = (
                                'listImgSize',
                                'historysize',
                                'browser',
                                'listBgColor',
                                'listFgColor',
                                'searchStop',
                                'groupItems',
                                'view',
                                'split',
                                'autosave',
                                'articles',
                                'lang',
                                'template',
                                'askImport',
                                'subject',
                                'toolbar',
                                'listImgSkin',
                                'columns',
                                'proxy',
                                'width',
                                'mailer',
                                'smtp',
                                'status',
                                'borrowers',
                                'confirm',
                                'itemWindowHeight',
                                'from',
                                'useRelativePaths',
                                'toolbarPosition',
                                'transform',
                                'itemWindowWidth',
                                'player',
                                'height',
                                'listBgPicture',
                                'emails',
                                'splash'
                            );
                            
        my @specificOptions = (
                                'sortOrder',
                                'details',
                                'sortField',
                                'plugin',
                                'multisite',
                              );
                              
        foreach (@commonOptions)
        {
            (my $gcfilmsOption = $_) =~ s/item/movie/;
            my $value = $options->$gcfilmsOption;
            $value =~ s|<br>|<br/>|gm;
            $self->$_($value);
        }
        my $newOptions = $self->{modelsFactory}->getModel('GCfilms')->{preferences};
        foreach (@specificOptions)
        {
            $newOptions->$_($options->$_);
        }
        
        # Special processing for order -> layout
        my $layout = 'form';
        my $order = $options->order;
        $layout = 'compact' if ($order == 1) || ($order == 2);
        $layout = 'readonly' if ($order == 3) || ($order == 4);
        $newOptions->layout($layout);

        $newOptions->save;
    }
    
    sub importGCfilmsData
    {
        my ($self, $options) = @_;
        $self->{parent}->importWithDetect($options->file, 1);
        $self->{options}->{file} = $ENV{GCS_DATA_HOME}.'/films.gcs';
        $self->{parent}->{items}->save($self->{parent});
        $self->{parent}->setFileName($self->{options}->{file});
    }
}

{
    package GCOptionsDialog;
    use GCPlugins;

    use Gtk2;
    use GCLang;
    use GCStyle;
    use GCMail;
    use base 'GCModalDialog';

    sub on_destroy
    {
        my ($widget, $self) = @_;
        return 1;
    }

    sub initValues
    {
        my $self = shift;
        
        $self->{browser}->setValue($self->{options}->browser);
        $self->{player}->setValue($self->{options}->player);
        $self->{audio}->setValue($self->{options}->audio);
        $self->{images}->setValue($self->{options}->images);
        $self->{confirm}->set_active($self->{options}->confirm);
        $self->{autosave}->set_active($self->{options}->autosave);
        $self->{autoload}->set_active(! $self->{options}->noautoload);
        $self->{splash}->set_active($self->{options}->splash);
        $self->{useRelativePaths}->set_active($self->{options}->useRelativePaths);
        $self->{status}->set_active($self->{options}->status);
        $self->{proxycb}->set_active($self->{options}->proxy);
		$self->{proxyurl}->set_text($self->{options}->proxy);
        $self->{transform}->set_active($self->{options}->transform);
        $self->{articles}->set_text($self->{options}->articles);

        if ($self->{options}->useTitleForPics)
        {
            $self->{picturesNameTitle}->set_active(1);
        }
        else
        {
            $self->{picturesNameAuto}->set_active(1);
        }

        $self->{options}->columns(3) if ! $self->{options}->exists('columns');
        $self->{columns} = $self->{options}->columns;

        $self->{options}->toolbar(1) if ! $self->{options}->exists('toolbar');
        $self->{toolbarOption}->set_history($self->{options}->toolbar);

        $self->{options}->toolbarPosition(0) if ! $self->{options}->exists('toolbarPosition');
        $self->{toolbarPositionOption}->set_history($self->{options}->toolbarPosition);

        $self->{options}->view(0) if ! $self->{options}->exists('view');
        $self->{viewOption}->set_history($self->{options}->view);
       
        my @plugins = @{$self->{plugins}};
        $self->{model}->{preferences}->plugin('ask')
            if ! $self->{model}->{preferences}->exists('plugin');
        my $i = 0;
        foreach (@plugins)
        {
            last if ($self->{model}->{preferences}->plugin eq 'ask')
                     || ($self->{model}->{preferences}->plugin eq 'multi')
                     || ($self->{model}->{preferences}->plugin eq 'multiask')
                     || ($_ eq $self->{model}->{preferences}->plugin);
            $i++;
        }
        $i++ if $self->{model}->{preferences}->plugin eq 'multiask';
        $i += 2 if $self->{model}->{preferences}->plugin eq 'multi';
        $self->{pluginOption}->set_history($i);

        $self->{askImport}->set_active($self->{options}->askImport);
        $self->{searchStop}->set_active($self->{options}->searchStop);
        #$self->{alwaysOriginal}->set_active($self->{options}->alwaysOriginal);
        
        $self->{langOption}->setValue($self->{options}->lang);
        $self->{styleOption}->setValue($self->{options}->style);

        my $layoutIdx = 0;
        foreach (@{$self->{model}->{panelsNames}})
        {
            last if $_ eq $self->{model}->{preferences}->layout;
            $layoutIdx++;
        }
        $self->{layoutOption}->set_history($layoutIdx);
        
        my @panelStyles= @{$self->{panelStyles}};
        my $panelStylesCurrent = $self->{options}->panelStyle;
        $i = 0;
        foreach (@panelStyles)
        {
            last if ($_ eq $panelStylesCurrent);
            $i++;
        }
        $self->{panelStyleOption}->set_history($i);

        $self->{options}->mailer('Sendmail') if ! $self->{options}->exists('mailer');
        $self->{mailerOption}->setValue($self->{options}->mailer);

        $self->{from}->set_text($self->{options}->from);
        
        $self->{options}->listImgSize(2) if ! $self->{options}->exists('listImgSize');
        $self->{listImgSize} = $self->{options}->listImgSize;
        $self->{options}->listImgSkin($GCStyle::defaultList) if ! $self->{options}->exists('listImgSkin');
        $self->{listImgSkin} = $self->{options}->listImgSkin;
        $self->{options}->listBgColor('65535,65535,65535') if ! $self->{options}->exists('listBgColor');
        $self->{options}->listFgColor('0,0,0') if ! $self->{options}->exists('listFgColor');
        $self->{mlbg} = $self->{options}->listBgColor;
        $self->{mlfg} = $self->{options}->listFgColor;
        $self->{listBgPicture} = $self->{options}->listBgPicture;
                
        $self->{proxyurl}->set_editable(0) if ! $self->{proxycb}->get_active;
        $self->{proxyurl}->set_editable(1) if $self->{proxycb}->get_active;

        $self->{historysize}->set_value($self->{options}->historysize);

        $self->{model}->{preferences}->details($self->{model}->{commonFields}->{title})
            if ! $self->{model}->{preferences}->exists('details');
        $self->{details} = $self->{model}->{preferences}->details;
        #$self->{options}->groupItems(1) if ! $self->{options}->exists('groupItems');
        #$self->{groupItems} = $self->{options}->groupItems;
        
        $self->{groupBy} = $self->{model}->{preferences}->groupBy;
        $self->{masterType} = $self->{model}->{preferences}->masterType;
        $self->{orderBy} = $self->{model}->{preferences}->orderBy;
    }
    
    sub activateInternetOptions
    {
        my ($self, $value) = @_;
        
        $self->{dataGroupLabel}->set_sensitive($value);
        $self->{pluginLabel}->set_sensitive($value);
        $self->{pluginOption}->set_sensitive($value);
        $self->{pluginList}->set_sensitive($value);
        $self->{askImport}->set_sensitive($value);
        $self->{searchStop}->set_sensitive($value);
    }
    
    sub show
    {
        my $self = shift;

        $self->initValues;
        
        $self->show_all;

        $self->{layoutOption}->signal_emit('changed');

        $self->{hboxImages}->hide;
        $self->{hboxDetails}->hide;
        $self->{hboxImages}->show_all if ($self->{options}->view == 1);
        $self->{hboxDetails}->show_all if ($self->{options}->view == 2);
        $self->activateInternetOptions(! $self->{model}->isPersonal);

        $self->{hboxSMTP}->hide if $self->{options}->mailer ne 'SMTP';
        $self->{pluginList}->hide
            if (($self->{model}->{preferences}->plugin ne 'multi')
             && ($self->{model}->{preferences}->plugin ne 'multiask'));

        my $code = $self->run;
        $self->hide;
        if ($code eq 'ok')
        {
            $self->{options}->browser($self->{browser}->getValue);
            $self->{options}->player($self->{player}->getValue);
            $self->{options}->audio($self->{audio}->getValue);
            $self->{options}->images($self->{images}->getValue);
	        $self->{options}->confirm(($self->{confirm}->get_active) ? 1 : 0);
	        $self->{options}->autosave(($self->{autosave}->get_active) ? 1 : 0);
	        $self->{options}->noautoload(($self->{autoload}->get_active) ? 0 : 1);
	        $self->{options}->status(($self->{status}->get_active) ? 1 : 0);
	        $self->{options}->splash(($self->{splash}->get_active) ? 1 : 0);
	        $self->{options}->useRelativePaths(($self->{useRelativePaths}->get_active) ? 1 : 0);
	        $self->{options}->useTitleForPics(($self->{picturesNameTitle}->get_active) ? 1 : 0);
	        $self->{options}->transform(($self->{transform}->get_active) ? 1 : 0);
            $self->{options}->articles($self->{articles}->get_text);

            if ($self->{proxycb}->get_active)
		    {
	            $self->{options}->proxy($self->{proxyurl}->get_text);
		    }
		    else
		    {
	            $self->{options}->proxy('');
		    }
	
            $self->{options}->lang($self->{langOption}->getValue);
            $self->{options}->style($self->{styleOption}->getValue);
            
            my $layoutIndex = $self->{layoutOption}->get_history;
            $self->{model}->{preferences}->layout($self->{model}->{panelsNames}->[$layoutIndex]);
            
            my $panelStyleIndex = $self->{panelStyleOption}->get_history;
            $self->{options}->panelStyle($self->{panelStyleOption}->child->get_label);

            $self->{options}->toolbar($self->{toolbarOption}->get_history);

            $self->{options}->toolbarPosition($self->{toolbarPositionOption}->get_history);

            my $currentView = $self->{options}->view;
            $self->{options}->view($self->{viewOption}->get_history);
            
            my $currentColumns = $self->{options}->columns;
            my $currentImgSize = $self->{options}->listImgSize;
            my $currentImgSkin = $self->{options}->listImgSkin;
            my $currentBg = $self->{options}->listBgColor;
            my $currentFg = $self->{options}->listFgColor;
            my $currentBgPicture = $self->{options}->listBgPicture;
            my $currentDetails = $self->{model}->{preferences}->details;
            #my $currentGroupItems = $self->{options}->groupItems;
            my $currentGroupBy = $self->{model}->{preferences}->groupBy;
            my $currentMasterType = $self->{model}->{preferences}->masterType;
            my $currentOrderBy = $self->{model}->{preferences}->orderBy;
            $self->{options}->columns($self->{columns});
            $self->{options}->listImgSize($self->{listImgSize});
            $self->{options}->listImgSkin($self->{listImgSkin});
            $self->{options}->listBgColor($self->{mlbg});
            $self->{options}->listFgColor($self->{mlfg});
            $self->{options}->listBgPicture($self->{listBgPicture});
            $self->{model}->{preferences}->details($self->{details});
            #$self->{options}->groupItems($self->{groupItems});
            $self->{model}->{preferences}->groupBy($self->{groupBy});
            $self->{model}->{preferences}->masterType($self->{masterType});
            $self->{model}->{preferences}->orderBy($self->{orderBy});
            
            if (
                 ($currentGroupBy ne $self->{model}->{preferences}->groupBy)
                 ||
                 ($currentMasterType ne $self->{model}->{preferences}->masterType)
                 ||
                 ($currentOrderBy ne $self->{model}->{preferences}->orderBy)
               )
            {
                $self->{parent}->{items}->setSerieFields;
            }
            
            
            if (
                 ($currentView != $self->{options}->view)
                 ||
                 (
                    ($self->{options}->view == 1)
                    &&
                    (
                      ($currentColumns != $self->{options}->columns)
                      ||
                      ($currentImgSize ne $self->{options}->listImgSize)
                      ||
                      ($currentImgSkin ne $self->{options}->listImgSkin)
                      ||
                      ($currentBg ne $self->{options}->listBgColor)
                      ||
                      ($currentFg ne $self->{options}->listFgColor)
                      ||
                      ($currentBgPicture != $self->{options}->listBgPicture)
                    )
                 )
                 ||
                 (
                    ($self->{options}->view == 2)
                    &&
                    (
                      ($currentImgSize ne $self->{options}->listImgSize)
                      ||
                      ($currentDetails ne $self->{model}->{preferences}->details)
                      ||
                      #($currentGroupItems != $self->{options}->groupItems)
                      #||
                      ($currentGroupBy ne $self->{model}->{preferences}->groupBy)
                      ||
                      ($currentMasterType ne $self->{model}->{preferences}->masterType)
                      ||
                      ($currentOrderBy ne $self->{model}->{preferences}->orderBy)
                    )
                 )
               )
            {
                $self->{parent}->setItemsList(0);
            }

		    my $selPlugin = $self->{pluginOption}->child->get_label;
		    my $plugin = 'ask';
		    foreach (@{$self->{plugins}})
		    {
				last if $selPlugin eq $self->{parent}->{lang}->{OptionsAskPlugins};
                last if $selPlugin eq $self->{parent}->{lang}->{OptionsPluginsMulti};
                last if $selPlugin eq $self->{parent}->{lang}->{OptionsPluginsMultiAsk};
				$plugin = $_;
				last if $plugin eq $selPlugin;
		    }
		    $plugin = 'multi' if $selPlugin eq $self->{parent}->{lang}->{OptionsPluginsMulti};
		    $plugin = 'multiask' if $selPlugin eq $self->{parent}->{lang}->{OptionsPluginsMultiAsk};
		    $self->{model}->{preferences}->plugin($plugin);
       
            $self->{options}->askImport(($self->{askImport}->get_active) ? 1 : 0);
            $self->{options}->searchStop(($self->{searchStop}->get_active) ? 1 : 0);
            #$self->{options}->alwaysOriginal(($self->{alwaysOriginal}->get_active) ? 1 : 0);

            $self->{options}->mailer($self->{mailerOption}->getValue);
            $self->{options}->smtp($self->{smtp}->get_text);
	        $self->{options}->from($self->{from}->get_text);
            
	        $self->{parent}->checkImagesDirectory(1);	       
            $self->{options}->historysize($self->{historysize}->get_value);
            $self->{parent}->{menubar}->{menuHistoryItem}->remove_submenu();
            $self->{parent}->{menubar}->{menuHistory} = Gtk2::Menu->new();
            $self->{parent}->{menubar}->addHistoryMenu();
            
            #$self->{model}->{preferences}->details($self->{details});
            
	        $self->{options}->save;
	    }
    }

    sub changeDetails
    {
        my $self = shift;
        my $parent = $self->{parent};
        my @tmpOptionsArray = split m/\|/, $self->{details};

        my $fieldsDialog = new GCDetailedOptionsDialog($self,
                                                       \@tmpOptionsArray);
        
        
        $fieldsDialog->show;
        $self->{details} = join('|', @{$self->{fields}});
        $fieldsDialog->destroy;
    }

    sub setModel
    {
        my ($self, $model) = @_;

        $self->{model} = $model;
        if ($self->{plugin})
        {
            $self->{pluginOption}->remove_menu;
            $self->{plugin}->destroy;
        }
        $self->{plugin} = new Gtk2::Menu();
		my @plugins = ($self->{parent}->{lang}->{OptionsAskPlugins},
                       $self->{parent}->{lang}->{OptionsPluginsMultiAsk},
                       $self->{parent}->{lang}->{OptionsPluginsMulti},
                       @{$model->getPluginsNames});
		foreach (@plugins)
		{
		    my $item = Gtk2::MenuItem->new_with_label($_);
		    $self->{plugin}->append($item) ;
		}
		$self->{plugins} = \@plugins;
		$self->{pluginOption}->set_menu($self->{plugin});
		$self->{plugin}->show_all;
		
		if ($self->{layout})
		{
            $self->{layoutOption}->remove_menu;
            $self->{layout}->destroy;
		}
        $self->{layout} = new Gtk2::Menu();
        foreach (@{$model->{panelsNames}})
        {
            my $text = $model->getDisplayedText($model->{panels}->{$_}->{label});
            my $item = Gtk2::MenuItem->new_with_label($text);
            $self->{layout}->append($item);
        }
        
        $self->{layoutOption}->set_menu($self->{layout}) ;
		
    }

    sub initMailerOption
    {
        my $self = shift;
        use locale;
        my @mailers = ({value => 'Sendmail', displayed => 'Sendmail'},
                       {value => 'SMTP', displayed => 'SMTP'});
        my $mailPrograms = GCMail::getMailers;
        foreach (sort keys %{$mailPrograms})
        {
            push @mailers, {value => $_, displayed => $_};
        }
        $self->{mailerOption}->setValues(\@mailers, 2);
    }

    sub new
    {
        my ($proto, $parent, $options) = @_;
        my $class = ref($proto) || $proto;
        my $self  = $class->SUPER::new($parent,
                                       $parent->{lang}->{OptionsTitle},
                                      );
        bless ($self, $class);

        $self->set_has_separator(0);

        $options = $parent->{options} if !$options;
        $self->{options} = $options;
        
        $self->{parent} = $parent;

        $self->{lang} = $parent->{lang};
        $self->{tooltips} = $parent->{tooltips};

        ################
        # Main options
        ################
        my $tableMain = new Gtk2::Table(2,2,0);
        $tableMain->set_row_spacings(10);
        $tableMain->set_col_spacings(20);
        $tableMain->set_border_width(0);
        
        my $labelLang = new GCLabel($parent->{lang}->{OptionsLang});
        $self->{langOption} = new GCMenuList;
        my @langValues;
        push @langValues, {value => $_, displayed => $GCLang::langs{$_}->{LangName}}
            foreach (keys %GCLang::langs);
        @langValues = sort {$a->{displayed} cmp $b->{displayed}} @langValues;
        $self->{langOption}->setValues(\@langValues);

        $tableMain->attach($labelLang, 0, 1, 1, 2, 'fill', 'fill', 0, 0);
        $tableMain->attach($self->{langOption}, 1, 2, 1, 2, 'fill', 'fill', 0, 0);

        my $labelStyle = new GCLabel($parent->{lang}->{OptionsStyle});
		$self->{styleOption} = new GCMenuList;
		my @styleValues;
		push @styleValues, {value => $_, displayed => $_} foreach (sort keys %GCStyle::styles);
		$self->{styleOption}->setValues(\@styleValues);
        $tableMain->attach($labelStyle, 0, 1, 2, 3, 'fill', 'fill', 0, 0);
        $tableMain->attach($self->{styleOption}, 1, 2, 2, 3, 'fill', 'fill', 0, 0);
	
        my $labelWarning = new Gtk2::Label;
        $labelWarning->set_markup('<b>'.$parent->{lang}->{OptionsWarning}.'</b>');
        $labelWarning->set_use_underline(1);
		$labelWarning->set_line_wrap(1);
		$labelWarning->set_justify('center');
        
        my $vboxMain = new Gtk2::VBox(0,0);
        $vboxMain->set_border_width(20);
        $vboxMain->pack_start($tableMain,1,1,0);
        #$vboxMain->pack_start(Gtk2::HSeparator->new,1,1,10);
        $vboxMain->pack_start($labelWarning,1,1,0);

        ##################
        # Display options
        ##################
        my $tableDisplay = new Gtk2::Table(9, 7, 0);
        $tableDisplay->set_row_spacings($GCUtils::halfMargin);
        $tableDisplay->set_col_spacings($GCUtils::margin);
        $tableDisplay->set_border_width($GCUtils::margin);
        
        my $labelDisplayInformationGroup = new GCHeaderLabel($parent->{lang}->{OptionsDisplayInformationGroup});
        $tableDisplay->attach($labelDisplayInformationGroup, 0, 6, 0, 1, 'fill', 'fill', 0, 0);

        my $labelView = new GCLabel($parent->{lang}->{OptionsView});
        $self->{viewOption} = new Gtk2::OptionMenu();
		$self->{view} = new Gtk2::Menu();
		my %views = %{$parent->{lang}->{OptionsViews}};
		foreach (0..(scalar(keys %views) - 1))
		{
		    my $item = Gtk2::MenuItem->new_with_label($views{$_});
		    $self->{view}->append($item);
		}
        $self->{viewOption}->set_menu($self->{view});
        $self->{viewOption}->set_alignment(0.0,0.0);
        $tableDisplay->attach($labelView, 2, 3, 1, 2, 'fill', 'fill', 0, 0);
        $tableDisplay->attach($self->{viewOption}, 3, 4, 1, 2, 'fill', 'fill', 0, 0);

        $self->{hboxImages} = new Gtk2::HBox(0,0);
        $self->{imagesOptionsDialog} = new GCImagesOptionsDialog($self);
        $self->{imagesButton} = new Gtk2::Button($parent->{lang}->{ImagesOptionsButton});
        $self->{imagesButton}->signal_connect('clicked', sub {
            $self->{imagesOptionsDialog}->show;
        });
        $self->{hboxImages}->pack_start($self->{imagesButton}, 0, 0, 0);

        $self->{hboxDetails} = new Gtk2::HBox(0,0);
        $self->{buttonDetails} = new Gtk2::Button($parent->{lang}->{ImagesOptionsButton});
        $self->{buttonDetails}->signal_connect('clicked' => sub {
            $self->changeDetails;
        });
        $self->{hboxDetails}->pack_start($self->{buttonDetails}, 0, 0, 0);

        $self->{viewOption}->signal_connect('changed' => sub {
                my %views = %{$self->{parent}->{lang}->{OptionsViews}};
                my $i = 0;
                $i++ while ($views{$i} ne $self->{viewOption}->child->get_label);
                $self->{hboxImages}->hide;
                $self->{hboxDetails}->hide;
                $self->{hboxImages}->show_all if ($i == 1);
                $self->{hboxDetails}->show_all if ($i == 2);
	        });
        $tableDisplay->attach($self->{hboxImages}, 4, 6, 1, 2, 'fill', 'fill', 0, 0);
        $tableDisplay->attach($self->{hboxDetails}, 4, 6, 1, 2, 'fill', 'fill', 0, 0);

        my $labelLayout = new GCLabel($parent->{lang}->{OptionsLayout});
		$self->{layoutOption} = new Gtk2::OptionMenu();
        $self->{layoutOption}->signal_connect('changed' => sub {
                my $layoutIndex = $self->{layoutOption}->get_history;
                my $panelName = $self->{model}->{panelsNames}->[$layoutIndex];
                my $readonly = ($self->{model}->{panels}->{$panelName}->{editable} eq 'false');
                if ($readonly)
                {
                    $self->{labelPanelStyle}->show;
                    $self->{panelStyleOption}->show;
                }
                else
                {
                    $self->{labelPanelStyle}->hide;
                    $self->{panelStyleOption}->hide;
                }
        });
        $tableDisplay->attach($labelLayout, 2, 3, 2, 3, 'fill', 'fill', 0, 0);
        $tableDisplay->attach($self->{layoutOption}, 3, 4, 2, 3, 'fill', 'fill', 0, 0);

        $self->{labelPanelStyle} = new GCLabel($parent->{lang}->{OptionsStyle});
        $self->{panelStyleOption} = new Gtk2::OptionMenu();
        my @panelStyles = sort @GCStyle::readOnlyStyles;
        $self->{panelStyle} = new Gtk2::Menu();
        foreach (@panelStyles)
        {
            my $item = Gtk2::MenuItem->new_with_label($_);
            $self->{panelStyle}->append($item);
        }
        $self->{panelStyles} = \@panelStyles;
        $self->{panelStyleOption}->set_menu($self->{panelStyle});
        $tableDisplay->attach($self->{labelPanelStyle}, 4, 5, 2, 3, 'fill', 'fill', 0, 0);
        $tableDisplay->attach($self->{panelStyleOption}, 5, 6, 2, 3, ['fill'], 'fill', 30, 0);

        my $labelToolbar = new GCLabel($parent->{lang}->{OptionsToolbar});
        $self->{toolbarOption} = new Gtk2::OptionMenu();
		$self->{toolbar} = new Gtk2::Menu();
		my %toolbars = %{$parent->{lang}->{OptionsToolbars}};
		foreach (0..(scalar(keys %toolbars) - 1))
		{
		    my $item = Gtk2::MenuItem->new_with_label($toolbars{$_});
		    $self->{toolbar}->append($item);
		}
        $self->{toolbarOption}->set_menu($self->{toolbar}) ;
        $tableDisplay->attach($labelToolbar, 2, 3, 3, 4, 'fill', 'fill', 0, 0);
        $tableDisplay->attach($self->{toolbarOption}, 3, 4, 3, 4, 'fill', 'fill', 0, 0);

        my $labelToolbarPosition = new GCLabel($parent->{lang}->{OptionsToolbarPosition});
        $self->{toolbarPositionOption} = new Gtk2::OptionMenu();
		$self->{toolbarPosition} = new Gtk2::Menu();
		my %toolbarPositions = %{$parent->{lang}->{OptionsToolbarPositions}};
		foreach (0..(scalar(keys %toolbarPositions) - 1))
		{
		    my $item = Gtk2::MenuItem->new_with_label($toolbarPositions{$_});
		    $self->{toolbarPosition}->append($item);
		}
        $self->{toolbarPositionOption}->set_menu($self->{toolbarPosition}) ;
        $tableDisplay->attach($labelToolbarPosition, 4, 5, 3, 4, 'fill', 'fill', 0, 0);
        $tableDisplay->attach($self->{toolbarPositionOption}, 5, 6, 3, 4, ['fill'], 'fill', 30, 0);

        $self->{status} = new Gtk2::CheckButton($parent->{lang}->{OptionsStatus});
        $self->{status}->set_active($options->status);
        $tableDisplay->attach($self->{status}, 2, 6, 4, 5, 'fill', 'fill', 0, 0);


        my $labelDisplayArticlesGroup = new GCHeaderLabel($parent->{lang}->{OptionsDisplayArticlesGroup});
        $tableDisplay->attach($labelDisplayArticlesGroup, 0, 6, 6, 7, 'fill', 'fill', 0, 0);

        $self->{transform} = new Gtk2::CheckButton($parent->{lang}->{OptionsTransform});
        $tableDisplay->attach($self->{transform}, 2, 6, 7, 8, 'fill', 'fill', 0, 0);
        my $labelArticles = new GCLabel($parent->{lang}->{OptionsArticles});
        $tableDisplay->attach($labelArticles, 2, 4, 8, 9, 'fill', 'fill', 0, 0); 
		$self->{articles} = new Gtk2::Entry;
		$self->{articles}->set_width_chars(10);
        $tableDisplay->attach($self->{articles}, 5, 7, 8, 9, ['expand', 'fill'], 'fill', 5, 0);
        $options->articles('le,la,les,l,un,une,des,a,the') if (! $options->exists('articles'));

        my $vboxDisplay = new Gtk2::VBox(0,0);
        $vboxDisplay->set_border_width(0);
        $vboxDisplay->pack_start($tableDisplay, 0, 0, 0);

        #################
        # Paths options
        #################
        my $vboxPath = new Gtk2::VBox(0,0);

        my $tablePath = new Gtk2::Table(11, 4, 0);
        $tablePath->set_row_spacings($GCUtils::halfMargin);
        $tablePath->set_col_spacings($GCUtils::margin);
        $tablePath->set_border_width($GCUtils::margin);

        my $labelProgramsGroup = new GCHeaderLabel($parent->{lang}->{OptionsPathProgramsGroup});
        $tablePath->attach($labelProgramsGroup, 0, 4, 0, 1, 'fill', 'fill', 0, 0);
        my $labelBrowser = new GCLabel($parent->{lang}->{OptionsBrowser});
        $tablePath->attach($labelBrowser, 2, 3, 1, 2, 'fill', 'fill', 0, 0);
        $self->{browser} = new GCFile($self, $self->{parent}->{lang}->{OptionsProgramTitle});
        $self->{browser}->setWidth(40);
        $tablePath->attach($self->{browser}, 3, 4, 1, 2, ['expand', 'fill'], 'fill', 0, 0);

        my $labelPlayer = new GCLabel($parent->{lang}->{OptionsPlayer});
        $tablePath->attach($labelPlayer, 2, 3, 2, 3, 'fill', 'fill', 0, 0);
        $self->{player} = new GCFile($self, $self->{parent}->{lang}->{OptionsProgramTitle});
        $self->{player}->setWidth(40);
        $tablePath->attach($self->{player}, 3, 4, 2, 3, ['expand', 'fill'], 'fill', 0, 0);

        my $labelAudio = new GCLabel($parent->{lang}->{OptionsAudio});
        $tablePath->attach($labelAudio, 2, 3, 3, 4, 'fill', 'fill', 0, 0);
        $self->{audio} = new GCFile($self, $self->{parent}->{lang}->{OptionsProgramTitle});
        $self->{audio}->setWidth(40);
        $tablePath->attach($self->{audio}, 3, 4, 3, 4, ['expand', 'fill'], 'fill', 0, 0);

        my $labelImagesGroup = new GCHeaderLabel($parent->{lang}->{OptionsPathImagesGroup});
        $tablePath->attach($labelImagesGroup, 0, 4, 5, 6, 'fill', 'fill', 0, 0);
        my $labelImages = new GCLabel($parent->{lang}->{OptionsImages});
        $tablePath->attach($labelImages, 2, 3, 6, 7, 'fill', 'fill', 0, 0);
        $self->{images} = new GCFile($self, $self->{parent}->{lang}->{OptionsImagesTitle}, 'select-folder');
        $self->{images}->setWidth(40);
        $tablePath->attach($self->{images}, 3, 4, 6, 7, ['expand', 'fill'], 'fill', 0, 0);
	
	    $self->{picturesNameFormat} = new GCLabel($parent->{lang}->{OptionsPicturesFormat});
        $tablePath->attach($self->{picturesNameFormat}, 2, 4, 7, 8, 'fill', 'fill', 0, 0);
        $self->{picturesNameAuto} = new Gtk2::RadioButton(undef, $parent->{lang}->{OptionsPicturesFormatInternal});
        $tablePath->attach($self->{picturesNameAuto}, 2, 4, 8, 9, 'fill', 'fill', 2*$GCUtils::margin, 0);
        $self->{picturesNameGroup} = $self->{picturesNameAuto}->get_group;
        $self->{picturesNameTitle} = new Gtk2::RadioButton($self->{picturesNameGroup}, $parent->{lang}->{OptionsPicturesFormatTitle});
        $tablePath->attach($self->{picturesNameTitle}, 2, 4, 9, 10, 'fill', 'fill', 2*$GCUtils::margin, 0);

	    $self->{useRelativePaths} = new Gtk2::CheckButton($parent->{lang}->{OptionsUseRelativePaths});
        $self->{useRelativePaths}->set_active($options->useRelativePaths);
        $self->{useRelativePaths}->set_active(0) if (! $options->exists('useRelativePaths'));
        $tablePath->attach($self->{useRelativePaths}, 2, 4, 10, 11, 'fill', 'fill', 0, 0);

        $vboxPath->pack_start($tablePath, 0, 0, 0);

        ###################
        # Internet options
        ###################
        my $tableInternet = new Gtk2::Table(10, 5, 0);
        $tableInternet->set_row_spacings($GCUtils::halfMargin);
        $tableInternet->set_col_spacings($GCUtils::margin);
        $tableInternet->set_border_width($GCUtils::margin);
	
        $self->{dataGroupLabel} = new GCHeaderLabel($parent->{lang}->{OptionsInternetDataGroup});
        $tableInternet->attach($self->{dataGroupLabel}, 0, 5, 0, 1, 'fill', 'fill', 0, 0); 
	    
		$self->{pluginLabel} = new GCLabel($parent->{lang}->{OptionsPlugins});
		$self->{pluginOption} = new Gtk2::OptionMenu();
		$self->{pluginList} = new Gtk2::Button($parent->{lang}->{OptionsPluginsList});
		$self->{pluginList}->signal_connect('clicked' => sub {
		  $parent->selectMultiPluginsList;
		});
		$self->{pluginOption}->signal_connect('changed' => sub {
	       if (($self->{pluginOption}->child->get_label eq $parent->{lang}->{OptionsPluginsMulti}) || ($self->{pluginOption}->child->get_label eq $parent->{lang}->{OptionsPluginsMultiAsk}))
           {
               $self->{pluginList}->show;
           }
           else
           {
               $self->{pluginList}->hide;
            }
	    });		
        $self->{askImport} = new Gtk2::CheckButton($parent->{lang}->{OptionsAskImport});
        $tableInternet->attach($self->{pluginLabel}, 2, 3, 1, 2, 'fill', 'fill', 0, 0); 
        $tableInternet->attach($self->{pluginOption}, 3, 4, 1, 2, 'fill', 'fill', 0, 0); 
        $tableInternet->attach($self->{pluginList}, 4, 5, 1, 2, 'fill', 'fill', 20, 0);
        $tableInternet->attach($self->{askImport}, 2, 5, 2, 3, 'fill', 'fill', 0, 0); 
        $self->{searchStop} = new Gtk2::CheckButton($parent->{lang}->{OptionsSearchStop});
        $tableInternet->attach($self->{searchStop}, 2, 5, 3, 4, 'fill', 'fill', 0, 0) if ($^O !~ /win32/i);

        my $settingsGroupLabel = new GCHeaderLabel($parent->{lang}->{OptionsInternetSettingsGroup});
        $tableInternet->attach($settingsGroupLabel, 0, 5, 5, 6, 'fill', 'fill', 0, 0); 
	
        $self->{proxycb} = new Gtk2::CheckButton($parent->{lang}->{OptionsProxy});
        $self->{proxycb}->set_active($options->proxy);
		$self->{proxyurl} = new Gtk2::Entry;
		$self->{proxyurl}->set_text($options->proxy);
        $self->{proxycb}->signal_connect('clicked' => sub {
	            if ($self->{proxycb}->get_active)
                {
                    $self->{proxyurl}->set_editable(1);
                }
                else
                {
                    $self->{proxyurl}->set_editable(0);
                }
	        });
        $tableInternet->attach($self->{proxycb}, 2, 3, 6, 7, 'fill', 'fill', 0, 0);
        $tableInternet->attach($self->{proxyurl}, 3, 4, 6, 7, 'fill', 'shrink', 0, 0);
        
        my $labelFrom = new GCLabel($self->{parent}->{lang}->{OptionsFrom});
        $self->{from} = new Gtk2::Entry;
        $tableInternet->attach($labelFrom, 2, 3, 7, 8, 'fill', 'fill', 0, 0);
        $tableInternet->attach($self->{from}, 3, 4, 7, 8, 'fill', 'fill', 0, 0);

        my $mailerLabel = new GCLabel($parent->{lang}->{OptionsMailer});
        $self->{mailerOption} = new GCMenuList;
        $self->initMailerOption;
        $tableInternet->attach($mailerLabel, 2, 3, 8, 9, 'fill', 'fill', 0, 0);
        $tableInternet->attach($self->{mailerOption}, 3, 4, 8, 9, 'fill', 'fill', 0, 0);
        
        $self->{hboxSMTP} = new Gtk2::HBox(0,0);
        my $SMTPLabel = new GCLabel($parent->{lang}->{OptionsSMTP});
        $self->{smtp} = new Gtk2::Entry;
		$self->{smtp}->set_text($options->smtp);
		$self->{smtp}->set_width_chars(20);
        $self->{hboxSMTP}->pack_start($SMTPLabel,0,0,0);
        $self->{hboxSMTP}->pack_start($self->{smtp},0,0,5);
        $tableInternet->attach($self->{hboxSMTP}, 4, 5, 8, 9, 'fill', 'fill', 0, 0);

        $self->{mailerOption}->signal_connect('changed' => sub {
	            if ('SMTP' eq $self->{mailerOption}->getValue)
                {
                    $self->{hboxSMTP}->show;
                }
                else
                {
                    $self->{hboxSMTP}->hide;
                }
	        });
        
        my $mailersButton = new GCButton($parent->{lang}->{OptionsConfigureMailers});
        my $hboxMua = new Gtk2::HBox(0,0);
        $hboxMua->pack_start($mailersButton,0,0,0);
        $tableInternet->attach($hboxMua, 2, 4, 9, 10, 'fill', 'fill', 0, 0);
        $mailersButton->signal_connect('clicked' => sub {
            my $dialog = $self->{parent}->getDialog('MailPrograms');
            $self->initMailerOption if $dialog->show;
        });

        my $vboxInternet = new Gtk2::VBox(0,0);
        $vboxInternet->set_border_width(0);
        $vboxInternet->pack_start($tableInternet,0,0,0);

        ###################
        # Features options
        ###################
        my $vboxConvenience = new Gtk2::VBox(0,0);
        my $tableFeature = new Gtk2::Table(10, 4, 0);
        $tableFeature->set_row_spacings($GCUtils::halfMargin);
        $tableFeature->set_col_spacings($GCUtils::margin);
        $tableFeature->set_border_width($GCUtils::margin);

        $self->{confirm} = new Gtk2::CheckButton($parent->{lang}->{OptionsRemoveConfirm});
        $self->{confirm}->set_active($options->confirm);
        $self->{autosave} = new Gtk2::CheckButton($parent->{lang}->{OptionsAutoSave});
        $self->{autosave}->set_active($options->autosave);
        $self->{autoload} = new Gtk2::CheckButton($parent->{lang}->{OptionsAutoLoad});
        $self->{autoload}->set_active(! $options->noautoload);
        $self->{splash} = new Gtk2::CheckButton($parent->{lang}->{OptionsSplash});
        $self->{splash}->set_active($options->splash);
        $self->{splash}->set_active(1) if (! $options->exists('splash'));

        my $conveniencesLabel = new GCHeaderLabel($parent->{lang}->{OptionsFeaturesConveniencesGroup});
        $tableFeature->attach($conveniencesLabel, 0, 4, 0, 1, 'fill', 'fill', 0, 0);
        $tableFeature->attach($self->{confirm}, 2, 4, 1, 2, 'fill', 'fill', 0, 0);
        $tableFeature->attach($self->{autosave}, 2, 4, 2, 3, 'fill', 'fill', 0, 0);
        $tableFeature->attach($self->{autoload}, 2, 4, 3, 4, 'fill', 'fill', 0, 0);
        $tableFeature->attach($self->{splash}, 2, 4, 4, 5, 'fill', 'fill', 0, 0);

        my $fileHistoryLabelText = $parent->{lang}->{MenuHistory};
        $fileHistoryLabelText =~ s/_//g;
        my $fileHistoryLabel = new GCHeaderLabel($fileHistoryLabelText);
        $tableFeature->attach($fileHistoryLabel, 0, 4, 7, 8, 'fill', 'fill', 0, 0);        

        my $labelHistorysize = new GCLabel($parent->{lang}->{OptionsHistory});
        my $adjHistory = Gtk2::Adjustment->new(0, 1, 20, 1, 1, 0) ;
        $self->{historysize} = Gtk2::SpinButton->new($adjHistory, 0, 0);
        $self->{buttonClearHistory} = new Gtk2::Button($parent->{lang}->{OptionsClearHistory});
        $self->{buttonClearHistory}->signal_connect('clicked' => sub {
            $self->{options}->history('');
        });
        $tableFeature->attach($labelHistorysize, 2, 3, 8, 9, 'fill', 'fill', 0, 0);
        $tableFeature->attach($self->{historysize}, 3, 4, 8, 9, 'fill', 'fill', 0, 0);
        $tableFeature->attach($self->{buttonClearHistory}, 2, 3, 9, 10, 'fill', 'fill', 0, 0);

        $vboxConvenience->pack_start($tableFeature,0,0,0);

        my $tabs = Gtk2::Notebook->new();
        $tabs->set_name('GCOptionsTabs');
        $tabs->set_tab_pos('left');
        $tabs->set_show_border(0);

        my ($mainButton, $displayButton, $pathButton, $internetButton, $conveniencesButton);

        $mainButton = GCImageBox->new_from_stock('gtk-home',
                                                  $parent->{lang}->{OptionsMain});
        $displayButton = GCImageBox->new_from_stock('gtk-select-color',
                                                     $parent->{lang}->{OptionsDisplay});
        $pathButton = GCImageBox->new_from_stock('gtk-directory',
                                                  $parent->{lang}->{OptionsPaths});
        $internetButton = GCImageBox->new_from_stock('gtk-network',
                                                      $parent->{lang}->{OptionsInternet});
        $conveniencesButton = GCImageBox->new_from_stock('gtk-properties',
                                                          $parent->{lang}->{OptionsConveniences});
        
        $tabs->append_page_menu($vboxMain, $mainButton, Gtk2::Label->new($parent->{lang}->{OptionsMain}));
        $tabs->append_page_menu($vboxDisplay, $displayButton, Gtk2::Label->new($parent->{lang}->{OptionsDisplay}));
        $tabs->append_page_menu($vboxPath, $pathButton, Gtk2::Label->new($parent->{lang}->{OptionsPaths}));
        $tabs->append_page_menu($vboxInternet, $internetButton, Gtk2::Label->new($parent->{lang}->{OptionsInternet}));
        $tabs->append_page_menu($vboxConvenience, $conveniencesButton, Gtk2::Label->new($parent->{lang}->{OptionsConveniences}));

        $tabs->set_tab_label_packing ($vboxMain, 1, 0, 'start');
        $tabs->set_tab_label_packing ($vboxDisplay, 1, 0, 'start');
        $tabs->set_tab_label_packing ($vboxPath, 1, 0, 'start');
        $tabs->set_tab_label_packing ($vboxInternet, 1, 0, 'start');
        $tabs->set_tab_label_packing ($vboxConvenience, 1, 0, 'start');

        $self->vbox->pack_start($tabs, 1, 1, 0);

        $self->{lang} = $parent->{lang};

        return $self;
    }

}

{
    # Class used to let user select images options
    package GCImagesOptionsDialog;
    use base "GCModalDialog";
    
    sub initValues
    {
        my $self = shift;
        
        $self->{columns}->set_value($self->{parent}->{columns});
        
        $self->{imgSizeOption}->set_history($self->{parent}->{listImgSize});
        
        my $idx = 0;
        foreach (@GCStyle::lists)
        {
            last if $_ eq $self->{parent}->{listImgSkin};
            $idx++;
        }
        $self->{optionStyle}->set_history($idx);
        
        $self->{listBgPicture}->set_active($self->{parent}->{listBgPicture});
        $self->activateColors(! $self->{parent}->{listBgPicture});
        $self->{mlbg} = $self->{parent}->{mlbg};
        $self->{mlfg} = $self->{parent}->{mlfg};
    }
    
    sub saveValues
    {
        my $self = shift;
        
        $self->{parent}->{columns} = $self->{columns}->get_value;
        
        $self->{parent}->{listImgSize} = $self->{imgSizeOption}->get_history;

        $self->{parent}->{listImgSkin} = $self->{optionStyle}->child->get_label;
        $self->{parent}->{mlbg} = $self->{mlbg};
        $self->{parent}->{mlfg} = $self->{mlfg};
        $self->{parent}->{listBgPicture} = (($self->{listBgPicture}->get_active) ? 1 : 0);
    }
    
    sub show
    {
        my $self = shift;

        $self->initValues;
        
        $self->show_all;
        my $code = $self->run;
        if ($code eq 'ok')
        {
            $self->saveValues;
        }
        $self->hide;
    }
    
    sub changeColor
    {
        my ($self, $type) = @_;
        
        my $dialog = new Gtk2::ColorSelectionDialog($self->{parent}->{lang}->{ImagesOptionsSelectColor});
        my $vboxPicture = new Gtk2::VBox(0,0);
        my @colors = split m/,/, $self->{'ml'.$type};
        my $previous = new Gtk2::Gdk::Color($colors[0], $colors[1], $colors[2]);
        $dialog->colorsel->set_current_color($previous) if $previous;
        my $response = $dialog->run;
        if ($response eq 'ok')
        {
            my $color = $dialog->colorsel->get_current_color;
            $self->{'ml'.$type} = join ',',$color->red, $color->green, $color->blue;
        }
        $dialog->destroy;
    }
    
    sub activateColors
    {
        my ($self, $value) = @_;
        
        $self->{labelStyle}->set_sensitive(!$value);
        $self->{optionStyle}->set_sensitive(!$value);
        $self->{labelBg}->set_sensitive($value);
        $self->{buttonBg}->set_sensitive($value);
        $self->{labelFg}->set_sensitive($value);
        $self->{buttonFg}->set_sensitive($value);
    }
    
    sub new
    {
        my ($proto, $parent) = @_;
        my $class = ref($proto) || $proto;
        
        my $self  = $class->SUPER::new($parent,
                                       $parent->{lang}->{ImagesOptionsTitle},
                                      );

        my $tableLayout = new Gtk2::Table(9,4);
        $tableLayout->set_row_spacings($GCUtils::halfMargin);
        $tableLayout->set_col_spacings($GCUtils::margin);
        $tableLayout->set_border_width($GCUtils::margin);

        my $labelColumns = new GCLabel($parent->{lang}->{OptionsColumns});
        my  $adj = Gtk2::Adjustment->new(0, 1, 10, 1, 1, 0) ;
        $self->{columns} = Gtk2::SpinButton->new($adj, 0, 0);

        my $imgSizeLabel = new GCLabel($parent->{lang}->{ImagesOptionsSizeLabel});
        $self->{imgSizeOption} = new Gtk2::OptionMenu;
		$self->{imgSize} = new Gtk2::Menu();
		my %imgSizes = %{$parent->{lang}->{ImagesOptionsSizeList}};
		foreach (0..(scalar(keys %imgSizes) - 1))
		{
		    my $item = Gtk2::MenuItem->new_with_label($imgSizes{$_});
		    $self->{imgSize}->append($item);
		}
        $self->{imgSizeOption}->set_menu($self->{imgSize}) ;

        $self->{listBgPicture} = new Gtk2::CheckButton($parent->{lang}->{ImagesOptionsBgPicture});
        $self->{listBgPicture}->signal_connect('clicked' => sub {
            $self->activateColors(! $self->{listBgPicture}->get_active);
        });
        
        $self->{labelStyle} = new GCLabel($parent->{lang}->{OptionsStyle});
		$self->{optionStyle} = new Gtk2::OptionMenu();
		$self->{style} = new Gtk2::Menu();
		foreach (@GCStyle::lists)
		{
		    my $item = Gtk2::MenuItem->new_with_label($_);
		    $self->{style}->append($item);
		}
		$self->{optionStyle}->set_menu($self->{style}) ;

       
        $self->{labelBg} = new GCLabel($parent->{lang}->{ImagesOptionsBg});
        $self->{buttonBg} = new Gtk2::Button($parent->{lang}->{ImagesOptionsSelectColor});
        $parent->{tooltips}->set_tip($self->{buttonBg},
                                     $parent->{lang}->{ImagesOptionsBgTooltip});
        $self->{buttonBg}->signal_connect('clicked' => sub {
            $self->changeColor('bg');
        });
        
        $self->{labelFg} = new GCLabel($parent->{lang}->{ImagesOptionsFg});
        $self->{buttonFg} = new Gtk2::Button($parent->{lang}->{ImagesOptionsSelectColor});
        $self->{buttonFg}->signal_connect('clicked' => sub {
            $self->changeColor('fg');
        });
        $parent->{tooltips}->set_tip($self->{buttonFg},
                                     $parent->{lang}->{ImagesOptionsFgTooltip});

        my $imagesDisplayGroupLabel = new GCHeaderLabel($parent->{lang}->{OptionsImagesDisplayGroup});
        $tableLayout->attach($imagesDisplayGroupLabel, 0, 4, 0, 1, 'fill', 'fill', 0, 0);
        $tableLayout->attach($labelColumns, 2, 3, 1, 2, 'fill', 'fill', 0, 0);
        $tableLayout->attach($self->{columns}, 3, 4, 1, 2, 'fill', 'fill', 0, 0);
        $tableLayout->attach($imgSizeLabel, 2, 3, 2, 3, 'fill', 'fill', 0, 0);
        $tableLayout->attach($self->{imgSizeOption}, 3, 4, 2, 3, 'fill', 'fill', 0, 0);

        my $imagesStyleGroupLabel = new GCHeaderLabel($parent->{lang}->{OptionsImagesStyleGroup});
        $tableLayout->attach($imagesStyleGroupLabel, 0, 4, 4, 5, 'fill', 'fill', 0, 0);
        $tableLayout->attach($self->{listBgPicture}, 2, 4, 5, 6, 'fill', 'fill', 0, 0);
        $tableLayout->attach($self->{labelStyle}, 2, 3, 6, 7, 'fill', 'fill', 0, 0);
        $tableLayout->attach($self->{optionStyle}, 3, 4, 6, 7, 'fill', 'fill', 0, 0);
        $tableLayout->attach($self->{labelBg}, 2, 3, 7, 8, 'fill', 'fill', 0, 0);
        $tableLayout->attach($self->{buttonBg}, 3, 4, 7, 8, 'fill', 'fill', 0, 0);
        $tableLayout->attach($self->{labelFg}, 2, 3, 8, 9, 'fill', 'fill', 0, 0);
        $tableLayout->attach($self->{buttonFg}, 3, 4, 8, 9, 'fill', 'fill', 0, 0);

        $tableLayout->show_all;

        $self->vbox->pack_start($tableLayout,1,1,0);

        bless ($self, $class);
        return $self;
    }
}

{
    # Class used to let user select detailed options
    package GCDetailedOptionsDialog;
    use GCDialogs;
    use base 'GCFieldsSelectionDialog';
    
    sub initValues
    {
        my $self = shift;
        
        $self->{imgSizeOption}->set_history($self->{parent}->{listImgSize});
        $self->{groupByOption}->setValue($self->{parent}->{groupBy});
        if ($self->{parent}->{masterType} eq 'generated')
        {
            $self->{masterAuto}->set_active(1);
        }
        else
        {
            $self->{masterDetermined}->set_active(1);
        }

#        my $i = 0;
#        foreach (@{$self->{fields}})
#        {
#            last if ($_ eq $self->{parent}->{orderBy});
#            $i++;
#        }
#        $self->{orderByOption}->set_history($i);
        $self->{orderByOption}->setValue($self->{parent}->{orderBy});
    }
    
    sub saveValues
    {
        my $self = shift;
        
        $self->{parent}->{listImgSize} = $self->{imgSizeOption}->get_history;

        #$self->{parent}->{groupItems} = (($self->{groupItems}->get_active) ? 1 : 0);

#        my $i = $self->{groupByOption}->get_history;
#        $self->{parent}->{groupBy} = '';
#        $self->{parent}->{groupBy} =  $self->{fields}->[$i - 1] if $i;
        $self->{parent}->{groupBy} =  $self->{groupByOption}->getValue;

        if ($self->{masterAuto}->get_active())
        {
            $self->{parent}->{masterType} = 'generated';
        }
        else
        {
            $self->{parent}->{masterType} = 'ordered';
        }

#        $self->{parent}->{orderBy} = $self->{fields}->[$self->{orderByOption}->get_history];
        $self->{parent}->{orderBy} = $self->{orderByOption}->getValue;
    }
    
    sub hideExtra
    {
        my $self = shift;

#        if ($self->{parent}->{model}->{masterType} eq 'empty')
#        {
#            $self->{orderLabel}->hide;
#            $self->{orderByOption}->hide;            
#        }
    }

    sub show
    {
        my $self = shift;

        $self->initValues;
        
        my $code = $self->SUPER::show;
        if ($code eq 'ok')
        {
            $self->saveValues;
        }
        $self->hide;
    }
    
    sub checkGroupOptions
    {
        my $self = shift;

        #my $sensitive = $self->{groupItems}->get_active;
        my $sensitive = ($self->{groupByOption}->getValue ne '');
        #$self->{groupByOption}->set_sensitive($sensitive);
        #$self->{orderLabel}->set_sensitive($sensitive);
        $self->{orderByOption}->set_sensitive($sensitive);
        $self->{masterLabel}->set_sensitive($sensitive);
        $self->{masterAuto}->set_sensitive($sensitive);
        $self->{masterDetermined}->set_sensitive($sensitive);
        
    }
    
    sub new
    {
        my ($proto, $parent, $preIdList) = @_;
        my $class = ref($proto) || $proto;
        
        my $self  = $class->SUPER::new($parent,
                                       $parent->{lang}->{DetailedOptionsTitle},
                                       $preIdList,
                                       1);

        bless ($self, $class);

        my $tableLayout = new Gtk2::Table(8,4);
        $tableLayout->set_border_width($GCUtils::margin);
        $tableLayout->set_col_spacings($GCUtils::margin);
        $tableLayout->set_row_spacings($GCUtils::halfMargin);

        $self->{groupItems} = new GCLabel($parent->{lang}->{DetailedOptionsGroupItems});
        $self->{groupByOption} = new GCFieldSelector;
        $self->{groupByOption}->setModel($parent->{parent}->{model});
        $self->{groupByOption}->signal_connect('changed' => sub {
            $self->checkGroupOptions;
        });
        $self->{orderByOption} = new GCFieldSelector;
        $self->{orderByOption}->setModel($parent->{parent}->{model});

        $self->{masterLabel} = new GCLabel($parent->{lang}->{DetailedOptionsMasterOption});
        $self->{masterAuto} = new Gtk2::RadioButton(undef, $parent->{lang}->{DetailedOptionsMasterGenerated});
        $self->{masterRadioGroup} = $self->{masterAuto}->get_group;
        $self->{masterDetermined} = new Gtk2::RadioButton($self->{masterRadioGroup}, $parent->{lang}->{DetailedOptionsMasterOrdered});
        
        my $imgSizeLabel = new GCLabel($parent->{lang}->{DetailedOptionsImageSize});
        $self->{imgSizeOption} = new Gtk2::OptionMenu();
		$self->{imgSize} = new Gtk2::Menu();
        my %imgSizes = %{$parent->{lang}->{ImagesOptionsSizeList}};
        foreach (0..(scalar(keys %imgSizes) - 1))
        {
            my $item = Gtk2::MenuItem->new_with_label($imgSizes{$_});
            $self->{imgSize}->append($item);
        }
        $self->{imgSizeOption}->set_menu($self->{imgSize}) ;

        my $preferencesLabel = new GCHeaderLabel($parent->{lang}->{OptionsDetailedPreferencesGroup});
        $tableLayout->attach($preferencesLabel, 0, 6, 0, 1, 'fill', 'fill', 0, 0);

        $tableLayout->attach($imgSizeLabel, 2, 3, 1, 2, 'fill', 'fill', 0, 0);
        $tableLayout->attach($self->{imgSizeOption}, 3, 4, 1, 2, 'fill', 'fill', 0, 0);

        $tableLayout->attach($self->{groupItems}, 2, 3, 2, 3, 'fill', 'fill', 0, 0);
        $tableLayout->attach($self->{groupByOption}, 3, 4, 2, 3, 'fill', 'fill', 0, 0);
        $tableLayout->attach($self->{masterLabel}, 2, 4, 3, 4, 'fill', 'fill', 0, 0);
        $tableLayout->attach($self->{masterAuto}, 2, 3, 4, 5, 'fill', 'fill', $GCUtils::margin, 0);
        $tableLayout->attach($self->{masterDetermined}, 2, 3, 5, 6, 'fill', 'fill', $GCUtils::margin, 0);
        $tableLayout->attach($self->{orderByOption}, 3, 4, 5, 6, 'fill', 'fill', 0, 0);

        my $fieldsLabel = new GCHeaderLabel($parent->{lang}->{DetailedOptionsFields});
        $tableLayout->attach($fieldsLabel, 0, 6, 7, 8, 'fill', 'fill', 0, 0);

        $self->vbox->pack_start($tableLayout,0,0,0);
        $self->vbox->remove($self->{marginBox});
        $self->vbox->reorder_child($tableLayout,0);

        $self->set_default_size(-1,480);

        return $self;
    }
}

{
    # Class used to let user select information to be displayed
    package GCDisplayOptionsDialog;
    use base "Gtk2::Dialog";
    
    sub show
    {
        my $self = shift;

        $self->initValues;
        
        $self->show_all;
        my $code = $self->run;
        if ($code eq 'ok')
        {
            $self->saveValues;
        }
        $self->hide;
    }
    
    sub new
    {
        my ($proto, $parent) = @_;
        my $class = ref($proto) || $proto;
        
        my $self  = $class->SUPER::new($parent->{lang}->{DisplayOptionsTitle},
                              $parent,
                              [qw/modal destroy-with-parent/],
                              @GCDialogs::okCancelButtons
                            );

        bless($self, $class);

        $self->{show} = {};
        $self->{options} = $parent->{model}->{preferences};
        $self->{parent} = $parent;

        $self->{lang} = $parent->{lang};
        
        return $self;
    }
    
    sub createContent
    {
        my ($self, $model) = @_;

        $self->{options} = $model->{preferences};
        if ($self->{tabs})
        {
            $self->vbox->remove($self->{hboxActions});
            $self->{hboxActions}->destroy;
            $self->vbox->remove($self->{tabs});
            $self->{tabs}->destroy;
        }

        $self->{fieldsInfo} = $model->getDisplayedInfo;
        $self->{fields} = $model->getFieldsCopy;
        
        #Add special items
        if (! $model->isPersonal)
        {
            unshift @{$self->{fields}}, 'searchButton';
            unshift @{$self->{fieldsInfo}->[0]->{items}},
                    {id => 'searchButton',
                     label => $model->getDisplayedText('PanelSearchButton')};
        }
        $self->createComponents;
        $self->initValues;

    }
    
    sub initValues
    {
        my $self = shift;
        $self->{show} = {};
        
        foreach (@{$self->{fields}})
        {
            my $isShown = 1;
            $isShown = 0 if ($self->{options}->hidden =~ m/\|$_\|/);
            $self->{show}->{$_} = $isShown;
            $self->{$_}->set_active($isShown);
        }
    }
    
    sub saveValues
    {
        my $self = shift;
        $self->{show} = {};
        
        my $hidden = '|';
        foreach (@{$self->{fields}})
        {
            my $isShown = 0;
            $isShown = 1 if $self->{$_}->get_active;
            $self->{show}->{$_} = $isShown;
            $hidden .= $_.'|' if !$isShown;
        }
        $self->{options}->hidden($hidden);
        $self->{options}->save;
    }
    
    sub selectAll
    {
        my $self = shift;
        
        foreach (@{$self->{fields}})
        {
            $self->{$_}->set_active(1);
        }
    }
    
    sub initBox
    {
        my ($self, $box, $values) = @_;
        $box->set_border_width(2);
        foreach (@$values)
        {
            $self->{$_->{id}} = new Gtk2::CheckButton($_->{label});
            $box->pack_start($self->{$_->{id}}, 0,0,5);
        }
    }
    
    sub createComponents
    {
        my $self = shift;
        
        $self->{tabs} = Gtk2::Notebook->new();
        $self->{tabs}->set_border_width(12);
        foreach (@{$self->{fieldsInfo}})
        {
            my $frame = new Gtk2::Frame();
            $frame->set_shadow_type('none');
            $frame->set_border_width($GCUtils::margin);
            $frame->set_label_align(1.0, 0.0);
            my $vbox = new Gtk2::VBox(0,0);
            $self->initBox($vbox, $_->{items});
            $frame->add($vbox);
            $self->{tabs}->append_page($frame, $_->{title});
        }
		
		$self->{hboxActions} = new Gtk2::HBox(0,0);
		my $allButton = new Gtk2::Button($self->{lang}->{DisplayOptionsAll});
		$allButton->signal_connect( clicked => sub {
		  $self->selectAll;
		});
		$self->{hboxActions}->pack_end($allButton,1,0,20);
		
		$self->vbox->pack_start($self->{tabs}, 1, 1, 2);
        $self->vbox->pack_start($self->{hboxActions}, 0, 0, 10);
    }
}

1;
