package GCPlugins::GCgames::GCJeuxVideoFr;

###################################################
#
#  Copyright 2005-2006 Tian
#
#  This file is part of GCstar.
#
#  GCstar is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  GCstar is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with GCstar; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###################################################

use strict;
use utf8;

use GCPlugins::GCgames::GCgamesCommon;

{
    package GCPlugins::GCgames::GCPluginJeuxVideoFr;

    use base 'GCPlugins::GCgames::GCgamesPluginsBase';

    sub start
    {
        my ($self, $tagname, $attr, $attrseq, $origtext) = @_;
	
        $self->{inside}->{$tagname}++;
        if ($self->{parsingList})
        {
            if ($tagname eq 'tr')
            {
                $self->{isGame} = 1
                    if $attr->{class} =~ /listel[1-2]/;
            }
            if ($self->{isGame})
            {
                if ($tagname eq 'img')
                {
                    if ($attr->{src} =~ m|/style/img/supv.*?([a-z0-9]*)\.gif|)
                    {
                        $self->{itemIdx}++;
                        $self->{itemsList}[$self->{itemIdx}]->{platform} = uc $1;
                    }
                }
                elsif ($tagname eq 'a')
                {
                    if ($attr->{href} =~ m|/fiche|)
                    {
                        $self->{itemsList}[$self->{itemIdx}]->{url} = $attr->{href};
                        $self->{isName} = 1;
                    }
                }
                elsif ($tagname eq 'td')
                {
                    $self->{isDate} = 1 if $attr->{class} eq 'datesortie';
                    $self->{isGenre} = 1 if $attr->{class} eq 'genre';
                }
            }
        }
        elsif ($self->{parsingTips})
        {
            if (
                (($tagname eq 'h2') && ($attr->{class} eq 'simple'))
                ||
                (($tagname eq 'div') && ($attr->{class} eq 'desc'))
               )
            {
                $self->{isTip} = 1;
            }
            
        }
        else
        {
            if ($tagname eq 'div')
            {
                if ($attr->{class} eq 'colleft')
                {
                    $self->{isInfo} = 1;
                    $self->{is} = '';
                }
                elsif ($attr->{class} eq 'titrejeu1')
                {
                    $self->{is} = 'name';
                }
                elsif ($attr->{class} eq 'photofiche')
                {
                    $self->{isBox} = 1;
                }
            }
            elsif ($tagname eq 'img')
            {
                if ($self->{isBox})
                {
                    $self->{curInfo}->{boxpic} = $attr->{src}
                        if $attr->{src} =~ /^http/;
                    $self->{isBox} = 0;
                }
                if ($self->{isScreen})
                {
                    $attr->{src} =~ /.*?id=([0-9]*)/;
                    my $image = "http://img.jeuxvideo.fr/photo/$1.jpg";
                    $self->{curInfo}->{screenshot2} = $image if !$self->{curInfo}->{screenshot2}
                                                             &&  $self->{curInfo}->{screenshot1};
                    $self->{curInfo}->{screenshot1} = $image if !$self->{curInfo}->{screenshot1};
                }
            }
            elsif ($tagname eq 'a')
            {
                $self->{isScreen} = 1
                    if $attr->{title} eq 'Toutes les Captures';
            }
        }
    }

    sub end
    {
		my ($self, $tagname) = @_;
		
        $self->{inside}->{$tagname}--;
        if ($self->{parsingList})
        {
            $self->{isGame} = 0
                if ($tagname eq 'tr');
        }
        elsif ($self->{parsingTips})
        {
        }
        else
        {
            $self->{isScreen} = 0 if $tagname eq 'a';
        }
    }

    sub text
    {
        my ($self, $origtext) = @_;

        if ($self->{parsingList})
        {
            if ($self->{isName})
            {
                $self->{itemsList}[$self->{itemIdx}]->{name} = $origtext;
                $self->{isName} = 0;
            }
            elsif ($self->{isDate})
            {
                $self->{itemsList}[$self->{itemIdx}]->{released} = $origtext;
                $self->{isDate} = 0;
            }
            elsif ($self->{isGenre})
            {
                $self->{itemsList}[$self->{itemIdx}]->{genre} = $origtext;
                $self->{isGenre} = 0;
            }
        }
        elsif ($self->{parsingTips})
        {
            if ($self->{isTip})
            {
                $self->{curInfo}->{secrets} .= "\n\n" if $self->{curInfo}->{secrets};
                $self->{curInfo}->{secrets} .= $origtext;
                $self->{isTip} = 0;
            }
        }
        else
        {
            if ($self->{isInfo})
            {
                $self->{is} = 'platform' if $origtext =~ /Plateforme/;
                $self->{is} = 'genre' if $origtext =~ /Genre/;
                $self->{is} = 'editor' if $origtext =~ /Editeur/;
                $self->{is} = 'players' if $origtext =~ /Mode de jeu/;
                $self->{is} = 'released' if $origtext =~ /Date de sortie/;
                $self->{isInfo} = 0;
            }
            elsif ($self->{is})
            {
                $origtext =~ s/^\s*//;
                if ($self->{is} eq 'players')
                {
                    $origtext =~ s/Exclusivement Solo/1/;
                    $origtext =~ s/\s*joueurs?//i;
                }
                
                $self->{curInfo}->{$self->{is}} = $origtext;
                $self->{is} = '';
            }
        }
    } 

    sub getTipsUrl
    {
        my $self = shift;
        
        my $url = $self->{curInfo}->{$self->{urlField}};
        $url =~ s/fiche-/astuce-/;
        return $url;
    }

    sub new
    {
        my $proto = shift;
        my $class = ref($proto) || $proto;
        my $self  = $class->SUPER::new();
        bless ($self, $class);

        $self->{hasField} = {
            name => 1,
            platform => 1,
            released => 1,
            genre => 1
        };

        $self->{isName} = 0;
        $self->{isGame} = 0;
        $self->{isDate} = 0;
        $self->{isGenre} = 0;
        $self->{isInfo} = 0;
        $self->{isBox} = 0;
        $self->{isScreen} = 0;
        $self->{isTip} = 0;
        $self->{is} = '';

        $self->{key} = {
                        b => 'Bas',
                        h => 'Haut',
                        g => 'Gauche',
                        d => 'Droite'
                       };

        return $self;
    }

    sub toKey
    {
        my ($self, $value) = @_;
        
        if (length($value) <= 2)
        {
            $value = substr($value, 0, 1);
        }
        elsif ($value =~ /_/)
        {
            $value =~ s/(.)_(.)/$self->{key}->{$1}.'-'.$self->{key}->{$2}/ge;
        }
        else
        {
            $value = ucfirst $value;            
        }
        
        return $value;
    }

    sub preProcess
    {
        my ($self, $html) = @_;

        if ($self->{parsingList})
        {
            $html =~ s|<br />|, |;
        }
        elsif ($self->{parsingTips})
        {
            $html =~ s|<b>(.*?)</b>|$1|g;
            $html =~ s|<img src='http://www\.jeuxvideo\.fr/style/img/man/bt_([a-z_]*)\.gif' align='absmiddle'>|$self->toKey($1)|ge;
        }

        return $html;
    }
    
    sub getSearchUrl
    {
		my ($self, $word) = @_;
	
        return 'http://www.jeuxvideo.fr/encyclopedie-jeux-video-ds-gba-gc-pc-ps2-psp-xbox-xbox360-ps3-rev---g0g-0-0-0.html?recherche='.$word;
    }
    
    sub getItemUrl
    {
		my ($self, $url) = @_;
		
        return 'http://www.jeuxvideo.fr' . $url;
    }

    sub getName
    {
        return 'jeuxvideo.fr';
    }
    
    sub getAuthor
    {
        return 'Tian';
    }
    
    sub getLang
    {
        return 'FR';
    }

    sub getCharset
    {
        my $self = shift;
    
        return "ISO-8859-1";
    }
}

1;
